/* AT&T DSP1600 Support Tools Manual, March 1992, Issue 2.0, Page 7-4
 *
 * 7.6.3 Example
 *
 * The following program is ana example of scaling for controlling
 * precision and overflow. It performs the cosine function, similar
 * to the subroutine in the DSP16A Application Library.
 *
 * The function is:
 *          cos(A) = z + c1z**3 + c2z**5 + c3z**7 + c4z**9
 * where
 *   z = PI/2 - |A|        prescaled(to increase precision of calc.)
 *   c1 = -0.1666666	   = -0.16665
 *   c2 = 0.83330251E-2    =  0.13333    shifted left by 4 bits
 *   c3 = -0.1980741E-3    = -0.40565    shifted left by 11 bits
 *   c4 = 0.2601903E-5     =  0.17051    shifted left by 16 bits
 *
 * The number A is passed to the subroutine as a signed, 16-bit, Q14
 * number, and is in the range  -PI/2 <= A <= PI/2.
 *
 * The number z will be in the positive part of that range. The maximum
 * values of the z terms and the Qformat necessary to encode them without
 * overflowing bit 31 follows:     (where n is for z**n)
 *	n	MAX	Q
 *	1	1.57	14
 *	2	2.46	13
 *	3	3.87	13
 *	5	9.56	11
 *	7	23.6	10
 *	9	58.22	9
 */

_cos: auc = 0
     pt = _cos1
     a0 = a0			/* set flags			*/
     if mi a0 = -a0		/* get absolute value of z	*/

     a0 = a0h+1.57079		/* calc PI/2 - |A|, Q14	format	*/

     a1 = a0			/* accumulate result in a1 - Q14*/
                y=a0
                x=a0
         p=x*y			/* z**2 in Q12			*/
     a0 = p
     a0 = a0<<1			/* z**2 in Q13			*/
                x=a0
         p=x*y
     a0 = p			/* z**3 in Q11 (Q13 * Q14)	*/
     a0 = a0<<2			/* z**3 in Q13			*/
     do 3 {
                y=a0		/* calculate z**5 in Q11,	*/
         p=x*y  *r0++=a0	/*    z**7 in Q9, z**9 in Q7	*/
     a0 = p
     a0 = a0<<1
     }
     *r0--
                y=a0  x=*pt++
         p=x*y  y=*r0--  x=*pt++  /* z4 * x**9			*/
     a0 = p
     a0 = a0>>7			  /* 16 right c4, 9 left z**9	*/
     a1 = a1+a0
         p=x*y  y=*r0--  x=*pt++  /* z3 * x**7			*/
     a0 = p
     a0 = a0>>4			  /* 11 right c3, 7 left z**7	*/
     a1 = a1+a0
         p=x*y  y=*r0--  x=*pt++  /* z2 * x**5			*/
     a0 = p
     a0 = a0<<1			  /* 4 right c2, 5 left z**5	*/
     a1 = a1+a0
         p=x*y  y=*r0--  x=*pt++  /* z1 * x**3			*/
     a0 = p
     a0 = a0<<3			  /* 0 right c1, 3 left z**3	*/
     a1 = a1+a0
     return
     
/*  To further explain the shifts in the last part, for example, the
 *  shift for the c2 * z5 term is as follows: the coeff is a Q14 number
 *               ( ^manual error showed c3 here and in code comment )
 *  by default, scaled by 4 bits. The z5 term is Q11. A Q14 times a Q11
 *  is Q9 (in 16 bits), hence it must be shifted left by five to get Q14.
 *  The right shift by four corrects for the original scaling of the
 *  coeff. The net shift is one left, and the result is Q14.
 */

